unit seznam;
{JEDNOSMERNY SEZNAM SE ZARAZKOU}
interface

uses typ;

type PPrvek = ^TPrvek;
     TPrvek = object         {prvek seznamu}
         data: T;
         dalsi: PPrvek;
         predch: PPrvek;
         constructor Init(d: T);
         function Hodnota: T;
         function TenDalsi: PPrvek;
         function TenPredch: PPrvek;
         procedure NastavHodnotu(d: T);
         procedure NastavDalsi(u: PPrvek);
         procedure NastavPredch(u: PPrvek);
     end;

     TSeznam = object           {Vlastni seznam}
        phlava, pzarazka: PPrvek;
        delka: integer;
        constructor Init;
        destructor Done;
        procedure chyba(s:string);
        procedure VlozNaZacatek(d: T);
        procedure VlozNaKonec(d: T);
        procedure VlozZa(kam: pprvek; d: T);
        procedure VlozPred(kam: pprvek; d: T);
        function AlokujPrvek(d: T): PPrvek;
        procedure Vyprazdni;
        function Najdi(d: T): PPrvek;
     end;

implementation


{********************* metody tridy prvek*****************}
constructor TPrvek.Init;
begin
   dalsi := nil;
   predch := nil;
   data := d;
end;


function TPrvek.hodnota;
begin
   hodnota := data;
end;

function TPrvek.TenDalsi;
begin
   TenDalsi := dalsi;
end;

function TPrvek.TenPredch;
begin
   TenPredch := predch;
end;


procedure TPrvek.NastavHodnotu;
begin
   data := d;
end;

procedure TPrvek.NastavDalsi;
begin
   dalsi := u;
end;

procedure TPrvek.NastavPredch;
begin
   predch := u;
end;

{************ metody tridy seznam ****************}
{konstruktor vytvori prazdny seznam, tj. alokuje hlavu a zarazku}
constructor TSeznam.INIT;
begin
   phlava := AlokujPrvek(0);
   pzarazka := AlokujPrvek(0);
   if (phlava = nil) or (pzarazka = nil)
   then chyba('chybi pamet v konstruktoru');
   phlava^.NastavDalsi(pzarazka);
   pzarazka^.NastavPredch(phlava);
   delka := 0;
end;

{destruktor zrusi vsechny prvky vcetne zarazky}
destructor TSeznam.done;
begin
  Vyprazdni;  {Odstran vsechny prvky krome zarazky}
  dispose(phlava); {Odstran hlavu a zarazku}
  dispose(pzarazka);
  phlava := nil;
  pzarazka := nil;
end;

{Bezpecne alokuje novy prvek seznamu; nejprve testuje, zda ma dost pameti}
{neni-li pamet, vola proceduru chyba}
function TSeznam.AlokujPrvek(d: T): PPrvek;
begin
    AlokujPrvek := nil;
    if maxavail > sizeof(TPrvek) then AlokujPrvek := new(PPrvek, Init(d));
end;


{vola se pri chybe: vypise upozorneni a ukonci program}
procedure TSeznam.chyba(S: string);
begin
  writeln;
  writeln('chyba: ', s);
  halt;
end;

{vytvori novy prvek a vlozi ho na zacatek seznamu}
procedure TSeznam.VlozNaZacatek;
begin
   VlozZa(phlava, d);
end;

{Vlozi novy prvek obsahujici d za prvek urceny adresou kam}
procedure TSeznam.VlozZa;
var p: PPrvek;
begin
    p := AlokujPrvek(d);
    if p = nil then chyba('malo pameti pro vkladani prvku');
    p^.NastavDalsi(kam^.TenDalsi);
    p^.NastavPredch(kam);
    kam^.NastavDalsi(p);
    p^.TenDalsi^.NastavPredch(p);
    inc(delka);
end;

{Vlozi novy prvek obsahujici d pred prvek urceny adresou kam}
procedure TSeznam.VlozPred;
var p: PPrvek;
begin
    p := AlokujPrvek(d);
    if p = nil then chyba('malo pameti pro vkladani prvku');
    p^.NastavDalsi(kam);
    p^.NastavPredch(kam^.TenPredch);
    kam^.NastavPredch(p);
    p^.TenPredch^.NastavDalsi(p);
    inc(delka);
end;

procedure TSeznam.VlozNaKonec(d: T);
begin
    VlozPred(pzarazka, d);
end;


{smaze vsechny prvky krome hlavy a zarazky}
procedure TSeznam.Vyprazdni;
var p,q: PPrvek;
begin
   p := phlava^.TenDalsi;
   q := p;
   while q <> pzarazka do begin    {smaze vsechny prvky krome zarazky a hlavy}
      p := p^.TenDalsi;
      dispose(q);
      q := p;
   end;
   pHlava^.NastavDalsi(pzarazka); {obnovi vazbu hlavy a zarazky}
   pZarazka^.NastavPredch(phlava);
   delka := 0;
end;


function TSeznam.Najdi;
var p: PPrvek;
begin
  p := phlava^.TenDalsi;
  pzarazka^.NastavHodnotu(d);
  while p^.Hodnota <> d do p := p^.TenDalsi;
  if p = pzarazka then najdi := nil
  else najdi := p;
end;

end.