(***********************************************************************
  	GRAF1.PAS
   druh verze; prvn pokus o vlastn implementaci pozdn vazby

  Objektov typy obsahuj navc atribut typ, kter obsahuje informaci
  o skutenm typu instance.
  Odvozen typy Kruh a Usecka nyn dd metody, kter jsme v minul verzi
  museli opisovat. Metody typu Bod, pro kter chceme uplatnit pozdn vazbu,
  nyn obsahuj navc voln metod potomk podle skutenho typu instance.

  Pi tomto een vynikne nesmyslnost volby BODU jako pedka USECKY
  a KRUZNICE: Pro m mt BOD takto podivn metody, kter se v potomcch
  mn? Copak je krunice zvltn ppad bodu?
***********************************************************************)

unit Graf1;

interface

uses Pomoc, Graph, Crt;

const cesta : string = 'c:\tp\bgi';

procedure Instrukce(text:string);

type
  Bod = object
     constructor Udelej(xx, yy: integer);
     destructor Zrus;
     procedure Zobraz(BARVA: integer);
     procedure Nakresli;
     procedure Smaz;
     procedure Obnov;
     procedure PosunSe(kam: Pohyb; oKolik: integer);
     procedure UsadSe(krok: integer);
    private
      x, y: integer;
      Sviti: byte;
      typ: byte;
  end;

  Kruh = object(Bod)
     constructor Udelej(xx, yy, rr: integer);
     procedure Zobraz(BARVA:integer);
     procedure PosunSe(kam: Pohyb; oKolik: integer);
    private
      r:  integer;
  end;

  Usecka = object(Bod)
     constructor Udelej(xx, yy:integer);
     procedure Zobraz(BARVA: integer);
     procedure PosunSe(kam: Pohyb; oKolik: integer);
    private
     polodelka: integer;
     u1, u2: real;
  end;

implementation

var ca, sa: real; {sinus a kosinus hlu otoen}
type pKruh = ^Kruh;
     pUsecka = ^Usecka;

{ ************************** metody typu Bod *************************** }

constructor Bod.Udelej(xx, yy: integer);  {inicializace objektu}
  begin
    x := xx;
    y := yy;
    Sviti := 0;
    typ := 1;
  end;

destructor Bod.Zrus;
  begin
     Smaz;
  end;

procedure Bod.Zobraz(Barva: integer);
  begin
    case typ of            {pokus o pozdn vazbu}
     1: PutPixel(x, y, BARVA);
     2: pKruh(@self)^.Zobraz(Barva);
     3: pUsecka(@self)^.Zobraz(Barva);
    end;
  end;

procedure Bod.Nakresli;
  begin
    Zobraz(WHITE);
    Sviti := 1;
  end;

procedure Bod.Obnov;
  begin
    if Sviti = 1 then Zobraz(WHITE);
  end;

procedure Bod.Smaz;
  begin
    Zobraz(BLACK);
    Sviti := 0;
  end;


procedure Bod.PosunSe(kam: Pohyb; oKolik: integer);
  begin
   case typ of        {pokus o pozdn vazbu}
   1:begin
       Smaz;
       case kam of
         Vlevo:  x := x-oKolik;
         Vpravo: x := x+oKolik;
         Nahoru: y := y-oKolik;
         Dolu:   y := y+oKolik;
       end;
       Nakresli;
     end;
   2: pKruh(@self)^.PosunSe(kam, oKolik);
   3: pUsecka(@self)^.PosunSe(kam, oKolik);
   end;
  end;

procedure Bod.UsadSe(krok: integer);
var Pokyn: Pohyb;
begin
    repeat
     Pokyn := CtiKlavesu;
     PosunSe(Pokyn, krok);
    until Pokyn = Konec;
end;

{ ************************** metody typu Kruh *************************** }

constructor Kruh.Udelej(xx, yy, rr: integer);
  begin
   Bod.Udelej(xx, yy);
   r := rr;
   typ := 2;
  end;

procedure Kruh.Zobraz(BARVA:integer);
  begin
    SetColor(BARVA);
    Circle(x,y,r);
  end;

procedure Kruh.PosunSe(kam: Pohyb; oKolik: integer);
  begin
     Smaz;
     case kam of
       Vlevo:  x := x-oKolik;
       Vpravo: x := x+oKolik;
       Nahoru: y := y-oKolik;
       Dolu:   y := y+oKolik;
       Zvetsi: r := r+oKolik;
       Zmensi: r := r-oKolik;
     end;
     if r < 0 then r := 0;
     Nakresli;
  end;


{ ************************** metody typu Usecka *************************** }

constructor Usecka.Udelej(xx, yy: integer);
  begin
   Bod.Udelej(xx, yy);
   polodelka := 50;
   u1 := 1.0;
   u2 := 0.0;
   typ := 3;
  end;

procedure Usecka.Zobraz(BARVA:integer);
  var x1, y1, x2, y2: integer;
  begin
    SetColor(BARVA);
    x1 := round(x+u1*polodelka);
    y1 := round(y+u2*polodelka);
    x2 := round(x-u1*polodelka);
    y2 := round(y-u2*polodelka);
    line(x1, y1, x2, y2);
  end;


procedure Usecka.PosunSe(kam: Pohyb; oKolik: integer);
  var t: real;
  begin
     Smaz;
     case kam of
       Vlevo:  x := x-oKolik;
       Vpravo: x := x+oKolik;
       Nahoru: y := y-oKolik;
       Dolu:   y := y+oKolik;
       Zvetsi: polodelka := polodelka+oKolik;
       Zmensi: polodelka := polodelka-oKolik;
       Otoc:  begin
         t := ca*u1 - sa*u2;
         u2 := sa*u1 + ca*u2;
         u1 := t;
        end;
     end;
     if polodelka < 0 then polodelka := 0;
     Nakresli;
  end;


{ ************************** samostatn procedury ************************** }

procedure Instrukce(text: string);
begin
  Settextstyle(Defaultfont, HorizDir, 1);
  SetViewPort(5, yy-50,xx-5,yy-5, ClipOn);
  ClearViewPort;
  moveto(50,5);
  OutText(text);
  SetViewPort(0, 0, xx, yy, true);
end;

{ ************************** inicializan st ************************** }

begin
 ca := cos(pi/10);
 sa := sin(pi/10);
end.