(***********************************************************************
  	GRAF1.PAS
  tet varianta, druh pokus o pozdn vazbu

  Nyn konen odstranme onen nesmysl s bodem jako spolenm pedkem
  pro dal brafick objekty. Bod, krunice i seka jsou zvltn ppady
  obecnjho pojmu "grafick objekt". Pouijeme pomocn typ GO, kter bude
  pedkem typ Bod, Kruh i Usecka.
  V nm deklarujeme ty metody, kter se v odvozench typech neli, a zddme
  je do potomk.
  Metody, pro kter chceme pouvat pozdn vazbu, definujeme jako "ist":
  budou obsahovat pouze mechanismus pozdn vazby, tedy voln metody potomka
  podle hodnoty pomocnho atributu typ.
  Tda GO je "abstraktn": nem smysl pouvat jej instance.
  Toto een je elegantnj ne pedchoz, nebo mechanismus pozdn vazby
  jsme skryli do "abstraktnho" pedka a metody potomk se mohou
  opravdu starat jen o to, co slibuje jejich jmno.
  Poznmka: hlavn program (PROGRAM.PAS) se prakticky nezmnil, pouze msto
  ukazatel na bod pouv ukazatele na GO.
*************************************************************************)

unit Graf1;

interface

uses Pomoc, Graph, Crt;

const cesta : string = 'd:\bp\bgi';

procedure Instrukce(text:string);

type              {abstraktn tda}
  GO = object
      constructor Udelej(xx, yy: integer);
      destructor Zrus;
      procedure Zobraz(BARVA: integer);
      procedure Nakresli;
      procedure Smaz;
      procedure Obnov;
      procedure PosunSe(kam: Pohyb; oKolik: integer);
      procedure UsadSe(krok: integer);
    private
      x, y: integer;
      Sviti: byte;
      typ: byte;
  end;


  Bod = object(GO)
     {constructor Udelej(xx, yy: integer);}
     procedure Zobraz(BARVA: integer);
     procedure PosunSe(kam: Pohyb; oKolik: integer);
  end;

  Kruh = object(GO)
     constructor Udelej(xx, yy, rr: integer);
     procedure Zobraz(BARVA:integer);
     procedure PosunSe(kam: Pohyb; oKolik: integer);
    private
      r:  integer;
  end;

  Usecka = object(GO)
     constructor Udelej(xx, yy:integer);
     procedure Zobraz(BARVA: integer);
     procedure PosunSe(kam: Pohyb; oKolik: integer);
    private
     polodelka: integer;
     u1, u2: real;
  end;

implementation

var ca, sa: real; {sinus a kosinus hlu otoen}
type pKruh = ^Kruh;
     pUsecka = ^Usecka;
     pBod = ^Bod;

{ ************************** metody typu GO************************* }
constructor GO.Udelej(xx, yy: integer);  {inicializace objektu}
  begin
    x := xx;
    y := yy;
    Sviti := 0;
    typ := 1;
  end;

destructor GO.Zrus;
  begin
     Smaz;
  end;

procedure GO.Zobraz(Barva: integer);
  begin                             {pozdn vazba}
    case typ of
     1: pBod(@self)^.Zobraz(Barva);
     2: pKruh(@self)^.Zobraz(Barva);
     3: pUsecka(@self)^.Zobraz(Barva);
     else begin
        writeln('chybn typ');
        Halt(2);
     end
    end;
  end;

procedure GO.PosunSe(kam: Pohyb; oKolik: integer);
  begin                                 {pozdn vazba}
   case typ of
   1: pBod(@self)^.PosunSe(kam, oKolik);
   2: pKruh(@self)^.PosunSe(kam, oKolik);
   3: pUsecka(@self)^.PosunSe(kam, oKolik);
   else begin
        writeln('chybn typ');
        Halt(2);
   end
   end;
  end;

procedure GO.Nakresli;
  begin
    Zobraz(WHITE);
    Sviti := 1;
  end;

procedure GO.Obnov;
  begin
    if Sviti = 1 then Zobraz(WHITE);
  end;

procedure GO.Smaz;
  begin
    Zobraz(BLACK);
    Sviti := 0;
  end;

procedure GO.UsadSe(krok: integer);
var Pokyn: Pohyb;
begin
    repeat
     Pokyn := CtiKlavesu;
     PosunSe(Pokyn, krok);
    until Pokyn = Konec;
end;

{ ************************** metody typu Bod *************************** }


procedure Bod.Zobraz(BARVA: integer);
  begin
      putpixel(x,y,BARVA);
  end;

procedure Bod.PosunSe(kam: Pohyb; oKolik: integer);
  begin
       Smaz;
       case kam of
         Vlevo:  x := x-oKolik;
         Vpravo: x := x+oKolik;
         Nahoru: y := y-oKolik;
         Dolu:   y := y+oKolik;
       end;
       Nakresli;
  end;

{ ************************** metody typu Kruh *************************** }

constructor Kruh.Udelej(xx, yy, rr: integer);
  begin
   inherited Udelej(xx, yy);
   r := rr;
   typ := 2;
  end;

procedure Kruh.Zobraz(BARVA:integer);
  begin
    SetColor(BARVA);
    Circle(x,y,r);
  end;

procedure Kruh.PosunSe(kam: Pohyb; oKolik: integer);
  begin
     Smaz;
     case kam of
       Vlevo:  x := x-oKolik;
       Vpravo: x := x+oKolik;
       Nahoru: y := y-oKolik;
       Dolu:   y := y+oKolik;
       Zvetsi: r := r+oKolik;
       Zmensi: r := r-oKolik;
     end;
     if r < 0 then r := 0;
     Nakresli;
  end;


{ ************************** metody typu Usecka *************************** }

constructor Usecka.Udelej(xx, yy: integer);
  begin
   inherited Udelej(xx, yy);
   polodelka := 50;
   u1 := 1.0;
   u2 := 0.0;
   typ := 3;
  end;

procedure Usecka.Zobraz(BARVA:integer);
  var x1, y1, x2, y2: integer;
  begin
    SetColor(BARVA);
    x1 := round(x+u1*polodelka);
    y1 := round(y+u2*polodelka);
    x2 := round(x-u1*polodelka);
    y2 := round(y-u2*polodelka);
    line(x1, y1, x2, y2);
  end;


procedure Usecka.PosunSe(kam: Pohyb; oKolik: integer);
  var t: real;
  begin
     Smaz;
     case kam of
       Vlevo:  x := x-oKolik;
       Vpravo: x := x+oKolik;
       Nahoru: y := y-oKolik;
       Dolu:   y := y+oKolik;
       Zvetsi: polodelka := polodelka+oKolik;
       Zmensi: polodelka := polodelka-oKolik;
       Otoc:  begin
         t := ca*u1 - sa*u2;
         u2 := sa*u1 + ca*u2;
         u1 := t;
        end;
     end;
     if polodelka < 0 then polodelka := 0;
     Nakresli;
  end;


{ ************************** samostatn procedury ************************** }

procedure Instrukce(text: string);
begin
  Settextstyle(Defaultfont, HorizDir, 1);
  SetViewPort(5, yy-50,xx-5,yy-5, ClipOn);
  ClearViewPort;
  moveto(50,5);
  OutText(text);
  SetViewPort(0, 0, xx, yy, true);
end;

{ ************************** inicializan st ************************** }

begin
 ca := cos(pi/10);
 sa := sin(pi/10);
end.